package com.iven.musicplayergo.adapters

import android.content.Context
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.TextView
import androidx.recyclerview.widget.RecyclerView
import com.afollestad.materialdialogs.MaterialDialog
import com.iven.musicplayergo.GoConstants
import com.iven.musicplayergo.R
import com.iven.musicplayergo.extensions.startSongFromQueue
import com.iven.musicplayergo.extensions.toFilenameWithoutExtension
import com.iven.musicplayergo.goPreferences
import com.iven.musicplayergo.helpers.DialogHelper
import com.iven.musicplayergo.helpers.ThemeHelper
import com.iven.musicplayergo.models.Music
import com.iven.musicplayergo.player.MediaPlayerHolder
import java.util.*

class QueueAdapter(
    private val ctx: Context,
    private val queueSongsDialog: MaterialDialog,
    private val mediaPlayerHolder: MediaPlayerHolder
) :
    RecyclerView.Adapter<QueueAdapter.QueueHolder>() {

    var queueSongs = mediaPlayerHolder.queueSongs
    private var mSelectedSong = mediaPlayerHolder.currentSong

    private val mDefaultTextColor =
        ThemeHelper.resolveColorAttr(ctx, android.R.attr.textColorPrimary)

    fun swapSelectedSong(song: Music?) {
        notifyItemChanged(queueSongs.indexOf(mSelectedSong.first))
        mSelectedSong = Pair(song, true)
        notifyItemChanged(queueSongs.indexOf(mSelectedSong.first))
    }

    fun swapQueueSongs(updatedQueueSongs: MutableList<Music>) {
        queueSongs = updatedQueueSongs
        notifyDataSetChanged()
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): QueueHolder {
        return QueueHolder(
            LayoutInflater.from(parent.context).inflate(
                R.layout.queue_item,
                parent,
                false
            )
        )
    }

    override fun getItemCount() = queueSongs.size

    override fun onBindViewHolder(holder: QueueHolder, position: Int) {
        holder.bindItems(queueSongs[holder.adapterPosition])
    }

    inner class QueueHolder(itemView: View) : RecyclerView.ViewHolder(itemView) {

        fun bindItems(song: Music) {

            itemView.run {

                val title = findViewById<TextView>(R.id.title)
                val duration = findViewById<TextView>(R.id.duration)
                val subtitle = findViewById<TextView>(R.id.subtitle)

                val displayedTitle =
                        if (goPreferences.songsVisualization != GoConstants.TITLE) {
                            song.displayName?.toFilenameWithoutExtension()
                        } else {
                            song.title
                        }

                title.text = displayedTitle

                when {
                    queueSongs.indexOf(mSelectedSong.first) == adapterPosition && mSelectedSong.second -> title.setTextColor(
                            ThemeHelper.resolveThemeAccent(ctx)
                    )
                    else -> title.setTextColor(mDefaultTextColor)
                }

                duration.text = DialogHelper.computeDurationText(ctx, song)

                subtitle.text =
                    context.getString(R.string.artist_and_album, song.artist, song.album)

                setOnClickListener {
                    mediaPlayerHolder.startSongFromQueue(song, mediaPlayerHolder.launchedBy)
                }
            }
        }
    }

    fun performQueueSongDeletion(adapterPosition: Int, title: TextView): Boolean {
        val song = queueSongs[adapterPosition]
        notifyItemChanged(adapterPosition)
        return if (title.currentTextColor != ThemeHelper.resolveThemeAccent(ctx)) {
            DialogHelper.showDeleteQueueSongDialog(
                ctx,
                Pair(song, adapterPosition),
                queueSongsDialog,
                this@QueueAdapter,
                mediaPlayerHolder
            )
            true
        } else {
            false
        }
    }
}
